#!/usr/bin/env python3
import paho.mqtt.subscribe as subscribe
import json
import time

# GX Device I.P Address
ip = '192.168.20.167'

# VRM Portal ID from GX device. 
VRMid = "b827eba69d10"

# MQTT Instance ID
MQTT_SolarCharger_ID = 288 # Victron Smart MPPT
MQTT_BatteryBMS_ID   = 4   # JBD BMS

def mqtt_request(mqtt_path):
    topic = subscribe.simple(mqtt_path, hostname=ip)
    data  = json.loads(topic.payload)
    topic = data['value']
    return topic


start = time.monotonic_ns()
SolarChargeLimit        = mqtt_request("N/"+VRMid+"/solarcharger/"+str(MQTT_SolarCharger_ID)+"/Settings/ChargeCurrentLimit")
SolarYield              = mqtt_request("N/"+VRMid+"/solarcharger/"+str(MQTT_SolarCharger_ID)+"/History/Daily/0/Yield")
BatteryCAPInst          = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/InstalledCapacity")
BatteryTemp             = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Dc/0/Temperature") 
MinCellVolts            = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MinCellVoltage")
MaxCellVolts            = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MaxCellVoltage")
MinCellVoltsID          = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MinVoltageCellId")
MaxCellVoltsID          = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MaxVoltageCellId")
CellVoltsDiff           = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Diff")
CellVoltsSum            = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Sum")
NumOfCells              = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/NrOfCellsPerBattery")
MaxChargeCurrentNOW     = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Info/MaxChargeCurrent")
MaxDischargeCurrentNOW  = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Info/MaxDischargeCurrent")
MaxChargeVoltage        = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Info/MaxChargeVoltage")
AllowedToCharge         = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Io/AllowToCharge")
AllowedToDischarge      = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Io/AllowToDischarge")
DVCCstatus              = mqtt_request("N/"+VRMid+"/system/0/Control/Dvcc")
Cell1V                  = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell1")
Cell2V                  = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell2")
Cell3V                  = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell3")
Cell4V                  = mqtt_request("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell4")

finish=time.monotonic_ns()
duration = finish -  start
print('\033[38;5;26m'f"Duration is {duration//1000000}ms"'\033[0m')


print(f"{SolarChargeLimit: <25} SolarChargeLimit")
print(f"{SolarYield: <25} SolarYield")
print(f"{BatteryCAPInst: <25} BatteryCAPInst")
print(f"{BatteryTemp: <25.1f} BatteryTemp")
print(f"{MinCellVolts: <25} MinCellVolts")
print(f"{MaxCellVolts: <25} MaxCellVolts")
print(f"{MinCellVoltsID: <25} MinCellVoltsID")
print(f"{MaxCellVoltsID: <25} MaxCellVoltsID")
print(f"{CellVoltsDiff: <25.3f} CellVoltsDiff")
print(f"{CellVoltsSum: <25.3f} CellVoltsSum")
print(f"{NumOfCells: <25} NumOfCells")
print(f"{MaxChargeCurrentNOW: <25} MaxChargeCurrentNOW")
print(f"{MaxDischargeCurrentNOW: <25} MaxDischargeCurrentNOW")
print(f"{MaxChargeVoltage: <25} MaxChargeVoltage")
print(f"{AllowedToCharge: <25} AllowedToCharge")
print(f"{AllowedToDischarge: <25} AllowedToDischarge")
print(f"{DVCCstatus: <25} DVCCstatus")
print(f"{Cell1V: <25} Cell1V")
print(f"{Cell2V: <25} Cell2V")
print(f"{Cell3V: <25} Cell3V")
print(f"{Cell4V: <25} Cell4V")
