#!/usr/bin/env python3

import paho.mqtt.client as mqtt
import time
import json
# GX Device I.P Address
ip = '192.168.20.167'

# VRM Portal ID from GX device. 
VRMid = "b827eba69d10"

# MQTT Instance ID
MQTT_SolarCharger_ID = 288 # Victron Smart MPPT
MQTT_BatteryBMS_ID   = 4   # JBD BMS

SolarChargeLimit        = None
SolarYield              = None
BatteryCAPInst          = None
BatteryTemp             = None
MinCellVolts            = None
MaxCellVolts            = None
MinCellVoltsID          = None
MaxCellVoltsID          = None
CellVoltsDiff           = None
CellVoltsSum            = None
NumOfCells              = None
MaxChargeCurrentNOW     = None
MaxDischargeCurrentNOW  = None
MaxChargeVoltage        = None
AllowedToCharge         = None
AllowedToDischarge      = None
DVCCstatus              = None
Cell1V                  = None
Cell2V                  = None
Cell3V                  = None
Cell4V                  = None

# Define callback functions
def on_connect(client, userdata, flags, rc):
    print("Connected with result code "+str(rc))
    topics = [("N/"+VRMid+"/solarcharger/"+str(MQTT_SolarCharger_ID)+"/Settings/ChargeCurrentLimit",0),
              ("N/"+VRMid+"/solarcharger/"+str(MQTT_SolarCharger_ID)+"/History/Daily/0/Yield",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/InstalledCapacity",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Dc/0/Temperature",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MinCellVoltage",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MaxCellVoltage",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MinVoltageCellId",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MaxVoltageCellId",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Diff",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Sum",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/NrOfCellsPerBattery",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Info/MaxChargeCurrent",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Info/MaxDischargeCurrent",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Info/MaxChargeVoltage",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Io/AllowToCharge",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Io/AllowToDischarge",0),
              ("N/"+VRMid+"/system/0/Control/Dvcc",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell1",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell2",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell3",0),
              ("N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell4",0),
              ]
    client.subscribe(topics)

def on_message(client, userdata, msg):

    global SolarChargeLimit, SolarYield, BatteryCAPInst, BatteryTemp, MinCellVolts, MaxCellVolts, MinCellVoltsID, \
    MaxCellVoltsID, CellVoltsDiff, CellVoltsSum, NumOfCells, MaxChargeCurrentNOW, MaxDischargeCurrentNOW, \
    MaxChargeVoltage, AllowedToCharge, AllowedToDischarge, DVCCstatus, Cell1V, Cell2V, Cell3V, Cell4V, start, finish
    
    if msg.topic == "N/"+VRMid+"/solarcharger/"+str(MQTT_SolarCharger_ID)+"/Settings/ChargeCurrentLimit":
        SolarChargeLimit = json.loads(msg.payload)
        SolarChargeLimit = SolarChargeLimit['value']
    
    elif msg.topic == "N/"+VRMid+"/solarcharger/"+str(MQTT_SolarCharger_ID)+"/History/Daily/0/Yield":
        SolarYield = json.loads(msg.payload)
        SolarYield = SolarYield['value']

    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/InstalledCapacity":
        BatteryCAPInst = json.loads(msg.payload)
        BatteryCAPInst = BatteryCAPInst['value']
#============================================================
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Dc/0/Temperature":
        BatteryTemp = json.loads(msg.payload)
        BatteryTemp = BatteryTemp['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MinCellVoltage":
        MinCellVolts = json.loads(msg.payload)
        MinCellVolts = MinCellVolts['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MaxCellVoltage":
        MaxCellVolts = json.loads(msg.payload)
        MaxCellVolts = MaxCellVolts['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MinVoltageCellId":
        MinCellVoltsID = json.loads(msg.payload)
        MinCellVoltsID = MinCellVoltsID['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/MaxVoltageCellId":
        MaxCellVoltsID = json.loads(msg.payload)
        MaxCellVoltsID = MaxCellVoltsID['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Diff":
        CellVoltsDiff = json.loads(msg.payload)
        CellVoltsDiff = CellVoltsDiff['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Sum":
        CellVoltsSum = json.loads(msg.payload)
        CellVoltsSum = CellVoltsSum['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/System/NrOfCellsPerBattery":
        NumOfCells = json.loads(msg.payload)
        NumOfCells = NumOfCells['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Info/MaxChargeCurrent":
        MaxChargeCurrentNOW = json.loads(msg.payload)
        MaxChargeCurrentNOW = MaxChargeCurrentNOW['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Info/MaxDischargeCurrent":
        MaxDischargeCurrentNOW = json.loads(msg.payload)
        MaxDischargeCurrentNOW = MaxDischargeCurrentNOW['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Info/MaxChargeVoltage":
        MaxChargeVoltage = json.loads(msg.payload)
        MaxChargeVoltage = MaxChargeVoltage['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Io/AllowToCharge":
        AllowedToCharge = json.loads(msg.payload)
        AllowedToCharge = AllowedToCharge['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Io/AllowToDischarge":
        AllowedToDischarge = json.loads(msg.payload)
        AllowedToDischarge = AllowedToDischarge['value']
    elif msg.topic == "N/"+VRMid+"/system/0/Control/Dvcc":
        DVCCstatus = json.loads(msg.payload)
        DVCCstatus = DVCCstatus['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell1":
        Cell1V = json.loads(msg.payload)
        Cell1V = Cell1V['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell2":
        Cell2V = json.loads(msg.payload)
        Cell2V = Cell2V['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell3":
        Cell3V = json.loads(msg.payload)
        Cell3V = Cell3V['value']
    elif msg.topic == "N/"+VRMid+"/battery/"+str(MQTT_BatteryBMS_ID)+"/Voltages/Cell4":
            Cell4V = json.loads(msg.payload)
            Cell4V = Cell4V['value']

# Create a client instance
client = mqtt.Client()

# Assign callback functions
client.on_connect = on_connect
client.on_message = on_message

# Connect to the broker
client.connect(ip, 1883, 60)

# Start the loop
client.loop_start()

while Cell4V is None:
    time.sleep(.01)

print(f"{SolarChargeLimit: <25} SolarChargeLimit")
print(f"{SolarYield: <25} SolarYield")
print(f"{BatteryCAPInst: <25} BatteryCAPInst")
print(f"{BatteryTemp: <25.1f} BatteryTemp")
print(f"{MinCellVolts: <25} MinCellVolts")
print(f"{MaxCellVolts: <25} MaxCellVolts")
print(f"{MinCellVoltsID: <25} MinCellVoltsID")
print(f"{MaxCellVoltsID: <25} MaxCellVoltsID")
print(f"{CellVoltsDiff: <25.3f} CellVoltsDiff")
print(f"{CellVoltsSum: <25.3f} CellVoltsSum")
print(f"{NumOfCells: <25} NumOfCells")
print(f"{MaxChargeCurrentNOW: <25} MaxChargeCurrentNOW")
print(f"{MaxDischargeCurrentNOW: <25} MaxDischargeCurrentNOW")
print(f"{MaxChargeVoltage: <25} MaxChargeVoltage")
print(f"{AllowedToCharge: <25} AllowedToCharge")
print(f"{AllowedToDischarge: <25} AllowedToDischarge")
print(f"{DVCCstatus: <25} DVCCstatus")
print(f"{Cell1V: <25} Cell1V")
print(f"{Cell2V: <25} Cell2V")
print(f"{Cell3V: <25} Cell3V")
print(f"{Cell4V: <25} Cell4V")
